class BaseNode(object):
    """
    The BaseNode class represents a basic building block used in OsintBuddy
    plugins. It is designed to implement the base styles used
    in other nodes that can render a nodes element
    with a specific element type, label, and style on the OSINTbuddy UI.

    label : str
        A string representing the label for the node.
    style : dict
        A dictionary representing the react style properties for the node.
    placeholder : str
        A string representing the placeholder for the node.

    _base_blueprint(self) -> dict:
        Returns a dictionary containing essential data for a node
        element type, such as the node type, label,
        placeholder, and style.
    """
    def __init__(self, **kwargs):
        self.label: str = ''
        self.style: dict = {}
        self.placeholder: str = ''

        for key, value in kwargs.items():
            if key == 'label' or key == 'style' or key == 'placeholder':
                setattr(self, key, value)

    def _base_blueprint(self):
        return {
            'type': self.node_type,
            'label': self.label,
            'placeholder': self.placeholder,
            'style': self.style
        }


class BaseInput(BaseNode):
    pass


class BaseDisplay(BaseNode):
    pass



#   ---------------------------------------------------------------------------------
#   Copyright (c) Microsoft Corporation. All rights reserved.
#   Licensed under the MIT License. See LICENSE in project root for information.
#   ---------------------------------------------------------------------------------
"""This is a sample python file for testing functions from the source code."""
from __future__ import annotations


def osintbuddy_test():
    """
    This defines the expected usage, which can then be used in various test cases.
    Pytest will not execute this code directly, since the function does not contain the suffex "test"
    """
    pass



from osintbuddy.node.base import BaseInput  # noqa
from osintbuddy.node.displays import (  # noqa
    Title,
    Text,
    CopyText,
    CopyCode,
    Json,
    Image,
    Video,
    Pdf,
    List,
    Table,
    Empty
)
from osintbuddy.node.inputs import (  # noqa
    UploadFileInput,
    TextInput,
    DropdownInput,
    NumberInput,
    DecimalInput,
)



from osintbuddy.node.base import BaseDisplay


class Title(BaseDisplay):
    node_type: str = 'title'

    def __init__(self, title='', subtitle='', text='', **kwargs):
        super().__init__(**kwargs)
        self.title: str = title
        self.subtitle: str = subtitle
        self.text: str = text

    def json(self):
        blueprint = self._base_blueprint()
        blueprint['title'] = self.title
        blueprint['subtitle'] = self.subtitle
        blueprint['text'] = self.text
        return blueprint


class Text(BaseDisplay):
    node_type: str = 'section'

    def __init__(self, value='', icon=None, **kwargs):
        super().__init__(**kwargs)
        self.value = value
        self.icon = icon

    def json(self):
        blueprint = self._base_blueprint()
        blueprint['value'] = self.value
        blueprint['icon'] = self.icon
        return blueprint


class Empty(BaseDisplay):
    node_type: str = 'empty'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class CopyText(BaseDisplay):
    node_type: str = 'copy-text'

    def __init__(self, value='', **kwargs):
        super().__init__(**kwargs)
        self.value = value

    def json(self):
        blueprint = self._base_blueprint()
        blueprint['value'] = self.value
        return blueprint


class CopyCode(BaseDisplay):
    node_type: str = 'copy-code'

    def __init__(self, value='', **kwargs):
        super().__init__(**kwargs)
        self.value = value

    def json(self):
        blueprint = self._base_blueprint()
        blueprint['value'] = self.value
        return blueprint


class Json(BaseDisplay):
    node_type: str = 'json'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class Image(BaseDisplay):
    node_type: str = 'image'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class Pdf(BaseDisplay):
    node_type: str = 'pdf'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class Video(BaseDisplay):
    node_type: str = 'video'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class List(BaseDisplay):
    node_type: str = 'list'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint


class Table(BaseDisplay):
    node_type: str = 'table'

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def json(self):
        blueprint = self._base_blueprint()
        return blueprint



import re
import unicodedata


# Slugify and related code is from the Django project, thanks guys!
# Project URL: https://github.com/django/django
# https://github.com/django/django/blob/main/django/utils/text.py
def slugify(value, allow_unicode=False):
    """
    Convert to ASCII if 'allow_unicode' is False. Convert spaces or repeated
    dashes to single dashes. Remove characters that aren't alphanumerics,
    underscores, or hyphens. Convert to lowercase. Also strip leading and
    trailing whitespace, dashes, and underscores.
    """
    value = str(value)
    if allow_unicode:
        value = unicodedata.normalize("NFKC", value)
    else:
        value = (
            unicodedata.normalize("NFKD", value)
            .encode("ascii", "ignore")
            .decode("ascii")
        )
    value = re.sub(r"[^\w\s-]", "", value.lower())
    return re.sub(r"[-\s]+", "-", value).strip("-_")


def to_camel_case(value: str):
    value_list = value.replace(' ', '_').lower().split('_')
    return value_list[0] + ''.join(e.title() for e in value_list[1:])


def to_snake_case(name):
    name = to_camel_case(name)
    name = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
    name = re.sub('__([A-Z])', r'_\1', name)
    name = re.sub('([a-z0-9])([A-Z])', r'\1_\2', name)
    return name.lower()



from typing import List, Any
from osintbuddy.node.base import BaseInput


class UploadFileInput(BaseInput):
    """
    The UploadFileInput class represents an upload file input node used
    in the OsintBuddy plugin system.
    value : Any
        The initial value for the input node.
    icon : str
        The icon to be displayed with the node.
    supported_files : List[str]
        A list of supported file extensions for the node.

    Usage Example:
    class Plugin(OBPlugin):
        node = [UploadFileInput(supported_files=['.pdf', '.docx'])]
    """
    node_type: str = "upload"

    def __init__(self, value="", supported_files=[], icon="IconFileUpload", **kwargs):
        super().__init__(**kwargs)
        self.value: Any = value
        self.icon: str = icon
        self.supported_files: List[str] = supported_files

    def json(self):
        node = self._base_blueprint()
        node["value"] = self.value
        node["icon"] = self.icon
        node["supported_files"] = self.supported_files
        return node


class TextInput(BaseInput):
    """The TextInput class represents a text input node used
    in the OsintBuddy plugin system.
    value : str
        The value stored in the element.
    icon : str
        The icon to be displayed with the input element.
    default : str
        The default value for the input element.

    Usage Example:
    class Plugin(OBPlugin):
        node = [TextInput(label='Email search', placeholder='Enter email')]
    """
    node_type: str = "text"

    def __init__(self, value="", default="", icon="IconAlphabetLatin", **kwargs):
        super().__init__(**kwargs)
        self.value: str = value
        self.icon: str = icon

    def json(self):
        node = self._base_blueprint()
        node["value"] = self.value
        node["icon"] = self.icon
        return node


class DropdownInput(BaseInput):
    """
    The DropdownInput class represents a dropdown menu node used
    in the OsintBuddy plugin system.
    options : List[any]
        A list of options for the dropdown menu.
    value : str
        The initially selected option in the dropdown menu.

    Usage Example:
    class Plugin(OBPlugin):
        node = [
            DropdownInput(
                options=[{'label': 'Option 1', 'tooltip': 'Hello on hover!'}],
                value='Option 1'
            )
        ]
    """
    node_type: str = "dropdown"

    def __init__(self, options=[], value={'label': '', 'tooltip': ''}, **kwargs):
        super().__init__(**kwargs)
        self.options: List[any] = options
        self.value: dict = value

    def json(self):
        node = self._base_blueprint()
        node["options"] = self.options
        node["value"] = self.value
        return node


class NumberInput(BaseInput):
    """
    The NumberInput class represents a whole number input node used
    in the OsintBuddy plugin system.

    value : int
        The integer value stored in the node.

    Usage Example:
    class Plugin(OBPlugin):
        node = [NumberInput(value=10, placeholder='Enter a whole number')]
    """
    node_type: str = "number"

    def __init__(self, value=1, icon="123", **kwargs):
        super().__init__(**kwargs)
        self.value: int = value
        self.icon = icon

    def json(self):
        node = self._base_blueprint()
        node["value"] = self.value
        node["icon"] = self.icon
        return node


class DecimalInput(BaseInput):
    """
    The DecimalInput class represents a decimal number input node used
    in the OsintBuddy plugin system.
    value : float
        The float value stored in the node.

    Usage Example:
    class Plugin(OBPlugin):
        node = [DecimalInput(value=3.14, placeholder='Enter a decimal number')]
    """
    node_type: str = "decimal"

    def __init__(self, value=3.14, icon="123", **kwargs):
        super().__init__(**kwargs)
        self.value: float = value
        self.icon = icon

    def json(self):
        node = self._base_blueprint()
        node["value"] = self.value
        node["icon"] = self.icon
        return node



# @todo

class PluginManager(object):
    def __init__(self, app):
        self.app = app
        self.plugins = []

    def load_routes(self):
        pass

    def load_plugin(self, plugin):
        pass



import os
import importlib
from typing import List, Any
from collections import defaultdict
# from osintbuddy.utils import slugify
from osintbuddy.node.base import BaseNode
from osintbuddy.errors import OBPluginError
from osintbuddy.utils import to_snake_case


class OBRegistry(type):
    plugins = []
    labels = []
    ui_labels = []

    def __init__(cls, name, bases, attrs):
        """
        Initializes the OBRegistry metaclass by adding the plugin class
        and its label if it is a valid plugin.
        """
        if name != 'OBPlugin' and issubclass(cls, OBPlugin):
            label = cls.label.strip()
            if cls.show_label is True:
                OBRegistry.ui_labels.append(label)
            else:
                OBRegistry.ui_labels.append(None)
            OBRegistry.labels.append(label)
            OBRegistry.plugins.append(cls)

    @classmethod
    def get_plugin(cls, plugin_label: str):
        """
        Returns the corresponding plugin class for a given plugin_label or
        'None' if not found.

        :param plugin_label: The label of the plugin to be returned.
        :return: The plugin class or None if not found.
        """
        for idx, label in enumerate(cls.labels):
            if label == plugin_label:
                return cls.plugins[idx]
        return None

    def __getitem__(self, i):
        return self.get_plugin[i]


def discover_plugins(
    dir_path: str = '/plugins.osintbuddy.com/src/osintbuddy/core/',
):
    """
    Scans the specified 'dir_path' for '.py' files, imports them as plugins,
    and populates the OBRegistry with classes.

    :param dir_path: The directory path where the plugins are located.
    :return: List of plugin classes
    """
    for r, _, files in os.walk(dir_path):
        for filename in files:
            modname, ext = os.path.splitext(filename)
            if ext == '.py':
                try:
                    modpath = r.replace("/app/", "")
                    if 'osintbuddy/core' in dir_path:
                        modpath = r.replace("/plugins.osintbuddy.com/src/", "")
                    modpath = modpath.replace("/", ".")
                    importlib.import_module(f'{modpath}{modname}')
                except ImportError as e:
                    print(f"Error importing plugin '{modpath}{modname}': {e}")

    return OBRegistry.plugins


def transform(label, icon='list', prompt=None):
    """
    A decorator add transforms to an osintbuddy plugin.

    Usage:
    @transform(label=<label_text>, icon=<tabler_react_icon_name>)
    def transform_to_ip(self, node, **kwargs):
        # Method implementation

    :param label: str, A string representing the label for the transform
        method, which can be utilized for displaying in the context menu.
    :param icon: str, Optional icon name, representing the icon associated
        displayed by the transform label. Default is "list".
    :return: A decorator for the plugin transform method.
    """
    def decorator_transform(func):
        def wrapper(self, node, **kwargs):
            return func(self=self, node=node, **kwargs)
        wrapper.label = label
        wrapper.icon = icon
        if prompt is not None:
            wrapper.prompt = prompt

        return wrapper
    return decorator_transform


class OBPlugin(object, metaclass=OBRegistry):
    """
    OBPlugin is the base class for all plugin classes in this application.
    It provides the required structure and methods for a plugin.
    """
    node: List[BaseNode]
    name: str = 'Plugin Node'
    color: str = '#145070'
    label: str = ''
    icon: str = 'atom-2'
    transform_icons: dict = {}
    show_label = True
    style: dict = {}

    def __init__(self):
        transforms = self.__class__.__dict__.values()
        self.transforms = {
            func.label: func for func in transforms if hasattr(func, 'label')
        }
        self.transform_labels = [
            {'label': func.label, 'icon': func.icon} for func in transforms
            if hasattr(func, 'icon') and hasattr(func, 'label')
        ]

    def __call__(self):
        return self.blueprint()

    @staticmethod
    def _map_node_elements(element, kwargs):
        label = to_snake_case(element['label'])
        for passed_label in kwargs:
            if passed_label == label:
                if type(kwargs[label]) is str:
                    element['value'] = kwargs[label]
                elif type(kwargs[label]) is dict:
                    for t in kwargs[label]:
                        element[t] = kwargs[label][t]
        return element

    @classmethod
    def blueprint(cls, **kwargs):
        """
        Generate and return a dictionary representing the plugins node.
        Includes label, name, color, icon, and a list of all elements
        for the node/plugin.
        """
        node = defaultdict(None)
        node['label'] = cls.label
        node['name'] = cls.name
        node['color'] = cls.color
        node['icon'] = cls.icon
        node['style'] = cls.style
        node['elements'] = []
        for element in cls.node:
            if type(element) is list:
                e = []
                for elm in element:
                    e.append(cls._map_node_elements(elm.json(), kwargs))
                node['elements'].append(e)
            else:
                e = cls._map_node_elements(element.json(), kwargs)
                node['elements'].append(e)
        return node

    def _get_transform(self, transform_type: str, node, **kwargs) -> Any:
        """ Return output from a function accepting node data.
            The function will be called with a single argument, the node data
            from when a node context menu action is taken - and should return
            a list of Nodes.
            None if the plugin doesn't provide a transform
            for the transform_type.
        """
        if self.transforms and self.transforms[transform_type]:
            try:
                transform = self.transforms[transform_type](
                    self=self,
                    node=node,
                    **kwargs
                )
                if not isinstance(transform, list):
                    return [transform]
                return transform
            except OBPluginError as e:
                raise OBPluginError(e)
        return None



from osintbuddy.plugins import Plugin
import socket

# @todo


class IpPlugin(Plugin):
    def __init__(self):
        self.transforms = {
            "geolocate": self.get_location
        }
        self.node_name = "IP Address"
        self.node_inputs = [
            {
                "placeholder": "...",
                "name": "input-name",
                "icon": "some-tabler-icon class",
                "type": str
            }
        ]

    def get_location(self, ip: str):
        return socket.getaddrinfo(ip)
